<?php if ( ! defined( 'ABSPATH' ) ) { exit; }

use Square\Exceptions\ApiException;
use Square\SquareClient;
use Square\Models\Money;
use Square\Models\CreatePaymentRequest;
use Square\Environment;

if ( class_exists ( 'WC_Payment_Gateway' ) ) {
	class WC_Cash_App_Pay_Gateway extends WC_Payment_Gateway {

		protected $SQ_Merchant_Id;
		protected $SQ_Refresh_Token;
		protected $SQ_Access_Token;
		protected $SQ_Sandbox_Merchant_Id;
		protected $SQ_Sandbox_Refresh_Token;
		protected $SQ_Sandbox_Access_Token;

		public function __construct() {

			$this->id = 'cash-app-pay'; // payment gateway plugin ID
			$this->icon = WCCASHAPP_PLUGIN_DIR_URL . 'assets/images/cashapp_35.png'; // URL of the icon that will be displayed on checkout page near your gateway name
			$this->has_fields = true; // in case you need a custom form
			$this->method_title = 'Cash App Pay by Square';
			$this->method_description = 'Cash App Pay is the official integration for Square merchants. You need to connect an existing Square merchant account or create one to fully integrate this payment method.<br>
			<p><strong>More details about this gateway can be found at <a href="https://square.theafricanboss.com" target="_blank">square.theafricanboss.com</a></strong></p>
			<p><a href="https://square.theafricanboss.com/signup" target="_blank">Sign up to become a Square merchant using our referral link</a></p>
			<p>You will receive free processing on up to $1,000 in credit card transactions for the first 180 days* and/or whatever their current offer at signup is that will show once you click on the link</p>
			<p>See how the <a href="' . admin_url('admin.php?page=wc_cashapp_compared') . '">Cash App Pay payment method compares to Cash App payment method</a></p>'; // will be displayed on the options page

			$this->init_settings();
			$this->enabled = $this->get_option( 'enabled' );
			$this->title = $this->get_option( 'checkout_title' ) ? $this->get_option( 'checkout_title' ) : $this->method_title;
			$this->description = $this->get_option( 'checkout_description' ) ? $this->get_option( 'checkout_description' ) : 'Use Cash App or your phone camera to scan the QR code to pay with Cash App';
			$this->SQ_Merchant_Id = $this->get_option( 'SQ_Merchant_Id' );
			$this->SQ_Refresh_Token = $this->get_option( 'SQ_Refresh_Token' );
			$this->SQ_Access_Token = $this->get_option( 'SQ_Access_Token' );
			$this->SQ_Location_Id = $this->get_option( 'SQ_Location_Id' );
			$this->enableSandbox = $this->get_option( 'enableSandbox' ) ?? 'no';
			$this->SQ_Sandbox_Merchant_Id = '';
			$this->SQ_Sandbox_Refresh_Token = '';
			$this->SQ_Sandbox_Access_Token = '';
			$this->SQ_Sandbox_Location_Id = '';
			$this->toggleTutorial = $this->get_option( 'toggleTutorial' );

			global $cashapp_fs;
			$upgrade_url = cashapp_fs()->get_upgrade_url();
			$pro = ' <a style="text-decoration:none" href="' .$upgrade_url . '" target="_blank"><sup style="color:red">PRO</sup></a>';
			$edit_with_pro = ' <a style="text-decoration:none" href="' .$upgrade_url . '" target="_blank">APPLY CHANGES WITH PRO</a>';

			$square = ' <a href="https://square.theafricanboss.com/access.php" target="_blank">Get it here</a>';
			$this->form_fields = array(
				'enabled' => array(
					'title'       => 'Enable Cash App Pay',
					'label'       => 'Check to Enable / Uncheck to Disable',
					'type'        => 'checkbox',
					'default'     => 'no'
				),
				'checkout_title'  => array(
					'title'       => 'Checkout Title',
					'type'        => 'text',
					'description' => 'This is the title which the user sees on the checkout page.',
					'default'     => 'Cash App Pay',
					'placeholder' => 'Cash App Pay',
				),
				'checkout_description' => array(
					'title'       => 'Checkout Notice',
					'type'        => 'textarea',
					'description' => 'This is the description which the user sees during checkout.',
					'default'     => 'Use Cash App or your phone camera to scan the QR code to pay with Cash App',
					'placeholder'     => 'Use Cash App or your phone camera to scan the QR code to pay with Cash App',
				),
				'SQ_Merchant_Id' => array(
					'title'       => 'Square Merchant ID<br>' . $square,
					'type'        => 'text',
					'description' => 'This is your Square Merchant ID.' . $square,
					'placeholder' => 'MXXXXXXXXXXXX',
				),
				'SQ_Refresh_Token' => array(
					'title'       => 'Square Refresh Token<br>' . $square,
					'type'        => 'password',
					'description' => 'This is your Square Refresh Token.' . $square,
					'placeholder' => 'token',
				),
				'SQ_Access_Token' => array(
					'title'       => 'Square Access Token<br>' . $square,
					'type'        => 'password',
					'description' => 'This is your Square Access Token.' . $square,
					'placeholder' => 'token',
				),
				'SQ_Location_Id' => array(
					'title'       => 'Square Location ID<br>' . $square,
					'type'        => 'text',
					'description' => 'This is your Square Location ID.' . $square,
					'placeholder' => 'LXXXXXXXXXXXX',
				),
				// 'enableSandbox' => array(
				// 	'title'       => 'Enable Sandbox for testing purposes',
				// 	'label'       => 'Check to Enable / Uncheck to Disable',
				// 	'type'        => 'checkbox',
				// 	'description' => 'This is for testing purposes only. Payments will not be processed.',
				// 	'default'     => 'no',
				// ),
				'toggleTutorial' => array(
					'title'       => 'Enable Tutorial on checkout',
					'label'       => 'Check to Enable / Uncheck to Disable',
					'type'        => 'checkbox',
					'description' => 'Help your customers checkout with ease',
					'default'     => 'no',
				),
			);

			// Gateways can support subscriptions, refunds, saved payment methods
			$this->supports = array('products');

			// This action hook saves the settings
			add_action( 'woocommerce_update_options_payment_gateways_' . $this->id, array( $this, 'process_admin_options' ) );

			// We need custom JavaScript to obtain a token
			add_action( 'wp_enqueue_scripts', array( $this, 'wc_cash_app_pay_payment_scripts' ) );

			// Thank you page
			add_action( 'woocommerce_thankyou_' . $this->id, array( $this, 'wc_cash_app_pay_thankyou_page' ) );

			// Admin Menu
			add_action('admin_menu', array( $this, 'wc_cash_app_pay_admin_menu' ) );
		}

		public function wc_cash_app_pay_admin_menu(){
			$capability = 'manage_options';
			$contact_url = "mailto:info@theafricanboss.com?subject=WC%20Cashapp%20Plugin%20Support&body=Hello%20Jean%2C%0D%0A%0D%0A";
			$cashapp_receipts = admin_url('edit.php?post_type=cashapp-receipts');
			global $cashapp_fs;
			$account_url = $cashapp_fs->get_account_url();

			$new = ' <sup style="color:#9f9">NEW</sup>';
			$square_parent_slug = 'wc-settings&tab=checkout&section=cash-app-pay';
			add_submenu_page( 'woocommerce', 'Cash App Pay by Square', 'Cash App Pay' . $new, 'manage_woocommerce', $square_parent_slug, null );

			add_menu_page( null, 'Cash App Pay' . $new, $capability , $square_parent_slug, array( $this, 'wc_cash_app_pay_admin_menu' ), 'dashicons-money-alt', 56 );
			add_submenu_page( $square_parent_slug , 'Square' , 'Square Tokens' . $new , $capability  , 'wc_cashapp_square' , array( $this, 'wc_cashapp_square_menu_page' ), null );
			add_submenu_page( $square_parent_slug , 'Compared' , 'Cash App vs Cash App Pay' . $new , $capability  , 'wc_cashapp_compared' ,  array( $this, 'wc_cashapp_compared_menu_page'), null );
			add_submenu_page( $square_parent_slug , 'Review Cash App Pay' , 'Review' , $capability  , 'https://wordpress.org/support/plugin/wc-cashapp/reviews/?filter=5' , null, null );
			add_submenu_page( $square_parent_slug , 'Account' , 'Account' , $capability  , $account_url , null , null );
			add_submenu_page( $square_parent_slug , 'Our Most Recommended Plugins' , 'Recommended Plugins' , $capability  , 'wc_cashapp_recommended' , array( $this, 'wc_cashapp_recommended_menu_page'), null );
			add_submenu_page( $square_parent_slug , 'Our Plugins' , 'Free Plugins' , $capability , admin_url("plugin-install.php?s=theafricanboss&tab=search&type=author") 	, null, null );
			add_submenu_page( $square_parent_slug , 'Help' , 'Help' , $capability  , 'wc_cashapp_help' , array( $this, 'wc_cashapp_help_menu_page'), null );
			add_submenu_page( $square_parent_slug , 'Contact' , 'Support' , $capability  , $contact_url , null , null );
		}

		public function wc_cashapp_square_menu_page() {
			require_once WCCASHAPP_PLUGIN_DIR . 'includes/admin/square.php';
		}
		public function wc_cashapp_recommended_menu_page() {
			require_once WCCASHAPP_PLUGIN_DIR . 'includes/admin/recommended.php';
		}

		public function wc_cashapp_help_menu_page() {
			require_once WCCASHAPP_PLUGIN_DIR . 'includes/admin/help.php';
		}

		public function wc_cashapp_compared_menu_page() {
			require_once WCCASHAPP_PLUGIN_DIR . 'includes/admin/compared.php';
		}


		// /**
		//  * Logging method.
		//  *
		//  * @param string $message Log message.
		//  * @param string $level Optional. Default 'info'. Possible values:
		//  *                      emergency|alert|critical|error|warning|notice|info|debug.
		//  */
		// public static function log( $message, $level = 'info' ) {
		// 	if ( self::$log_enabled ) {
		// 		if ( empty( self::$log ) ) {
		// 			self::$log = wc_get_logger();
		// 		}
		// 		self::$log->log( $level, $message, array( 'source' => 'cash_app_pay' ) );
		// 	}
		// }

		// /**
		//  * Check if this gateway is available in the user's country based on currency.
		//  *
		//  * @return bool
		//  */
		// public function is_valid_for_use() {
		// 	return in_array(
		// 		get_woocommerce_currency(),
		// 		apply_filters(
		// 			'woocommerce_cash_app_pay_supported_currencies',
		// 			array( 'AUD', 'BRL', 'CAD', 'MXN', 'NZD', 'HKD', 'SGD', 'USD', 'EUR', 'JPY', 'TRY', 'NOK', 'CZK', 'DKK', 'HUF', 'ILS', 'MYR', 'PHP', 'PLN', 'SEK', 'CHF', 'TWD', 'THB', 'GBP', 'RMB', 'RUB', 'INR' )
		// 		),
		// 		true
		// 	);
		// }

		// /**
		//  * Admin Panel Options.
		//  * - Options for bits like 'title' and availability on a country-by-country basis.
		//  *
		//  * @since 1.0.0
		//  */
		// public function admin_options() {
		// 	if ( $this->is_valid_for_use() ) {
		// 		parent::admin_options();
		// 	} else {
		 		// echo '<div class="inline error"><p><strong>' . esc_html_e( 'Gateway disabled', 'wc-cashapp' ) . '</strong>:' .
				// 		esc_html_e( 'Cash App Pay does not support your store currency.', 'wc-cashapp' ) .
				// 		'</p></div>';
			// }
		// }

		// Payment Custom JS and CSS
		public function wc_cash_app_pay_payment_scripts() {
			if ( is_checkout() ) {
				// WCCASHAPP_PLUGIN_DIR_URL . 'assets/css/spinner.css'
				if ($this->enableSandbox == 'yes') {
					$square_web = "https://sandbox.web.squarecdn.com/v1/square.js";
				} else {
					$square_web = "https://web.squarecdn.com/v1/square.js";
				}
				wp_enqueue_script( 'square_web', $square_web );

				// $square_ver  = date("ymd-Gis", filemtime( plugin_dir_path( __FILE__ ) . 'assets/js/square.js' ));
				// $spinner_ver = date("ymd-Gis", filemtime( plugin_dir_path( __FILE__ ) . 'assets/css/spinner.css' ));

				// wp_enqueue_script( 'square', plugins_url( 'assets/js/square.js', array(), $square_ver );
				// wp_register_style( 'spinner', plugins_url( 'assets/css/spinner.css', false, $spinner_ver );

				wp_enqueue_script( 'square', WCCASHAPP_PLUGIN_DIR_URL . 'assets/js/square.js' );
				wp_register_style( 'spinner', WCCASHAPP_PLUGIN_DIR_URL . 'assets/css/spinner.css' );
				wp_enqueue_style ( 'spinner' );
			}
		}

		// Checkout page
		public function payment_fields () {
			global $woocommerce, $total, $amount;
			$woocommerce->cart->get_cart();
			$total = $woocommerce->cart->get_total(); // $1.00
			$amount = $woocommerce->cart->total; // 1.00
			$sq_environment = $this->enableSandbox == 'yes' ? Environment::SANDBOX : Environment::PRODUCTION;
			$sq_location = $this->enableSandbox == 'yes' ? $this->SQ_Sandbox_Location_Id : $this->SQ_Location_Id;

			echo '<fieldset id="wc-' , esc_attr( $this->id ) , '-form">';
			do_action( 'woocommerce_form_start' , $this->id );

			if ( !empty($this->checkout_description) ) {
				echo '<p>' , wp_kses_post( __( $this->checkout_description, 'wc-cashapp' ) ) . '</p>';
			} else {
				echo '<p>' , wp_kses_post( __( 'Use Cash App or your phone camera to scan the QR code to pay with Cash App', 'wc-cashapp' ) ) . '</p>';
			}

			// // as seen on squareup.com/shop/hardware/checkout
			// echo `
			// 	<div id="pay-with-cash-qr-code" class="jss20">
			// 		<div class="sq-cash-qr-container">
			// 			<div class="col-left">
			// 				<div class="sq-cash-qr-instruction">
			// 					<div>
			// 						<h3>Cash App Pay</h3><span>Scan the QR Code to check out</span>
			// 					</div>
			// 				</div>
			// 				<div class="sq-cash-qr-guide">
			// 					<img src="data:image/svg+xml,%3Csvg width='133' height='109' viewBox='0 0 133 109' fill='none' xmlns='http://www.w3.org/2000/svg'%3E%3Cg clip-path='url(%23clip0)'%3E%3Crect x='6.51207' y='2.51207' width='122.976' height='125.976' rx='14.9276' fill='%2300B843' stroke='black' stroke-width='4.97585'/%3E%3Cg clip-path='url(%23clip1)'%3E%3Cpath d='M59.7669 79.5683C60.3043 79.5683 60.6028 79.2399 60.6028 78.6727V78.1054C64.7527 77.8069 67.1112 75.0602 67.1112 71.7164C67.1112 67.5964 63.5883 66.1037 60.6028 65.3275V59.5953C61.7373 59.8341 62.633 60.3118 63.4391 61.1179C63.8272 61.506 64.2451 61.5956 64.693 61.297L65.8872 60.5506C66.3947 60.2521 66.5141 59.7744 66.126 59.3266C64.693 57.595 62.6927 56.6695 60.6028 56.4306V55.6843C60.6028 55.2066 60.3043 54.9677 59.7669 54.9677C59.2295 54.9677 58.9011 55.2961 58.9011 55.8634V56.3709C55.1692 56.5202 52.4524 58.7892 52.4524 62.4315C52.4524 66.7008 56.244 68.104 58.9011 68.7309V75.0005C57.5277 74.8512 56.1544 74.284 55.1095 72.8509C54.7512 72.3434 54.393 72.224 53.8854 72.5524L52.4822 73.4182C51.9747 73.7466 51.8553 74.1646 52.1837 74.6423C53.6764 76.6724 56.0051 77.9263 58.9011 78.1353V78.8518C58.9011 79.2996 59.2295 79.5683 59.7669 79.5683ZM56.035 62.133C56.035 60.4312 57.3486 59.5953 58.9011 59.4759V64.9095C57.2889 64.4617 56.035 63.775 56.035 62.133ZM63.6182 72.0449C63.6182 73.7167 62.5434 74.8811 60.6028 75.0602V69.1489C62.4837 69.6863 63.6182 70.3431 63.6182 72.0449Z' fill='white'/%3E%3Cpath d='M84.4025 67.1785C84.4025 61.4761 82.4619 56.1022 76.8491 56.1022C71.2065 56.1022 69.2958 61.4761 69.2958 67.1785C69.2958 72.8509 71.2065 78.195 76.8491 78.195C82.4619 78.195 84.4025 72.8509 84.4025 67.1785ZM80.372 67.1486C80.372 71.5672 79.5361 74.9408 76.8491 74.9408C74.1025 74.9408 73.2964 71.5672 73.2964 67.1486C73.2964 62.7301 74.1025 59.3863 76.8491 59.3863C79.5361 59.3863 80.372 62.7301 80.372 67.1486Z' fill='white'/%3E%3C/g%3E%3Cg filter='url(%23filter0_d)'%3E%3Ccircle cx='21.7048' cy='28.7714' r='21.7048' fill='%2300D64F'/%3E%3C/g%3E%3Cpath fill-rule='evenodd' clip-rule='evenodd' d='M15.6143 24.2473C15.5832 24.6287 15.5824 25.1239 15.5824 25.8616V26.8927H13.6492V25.8616V25.8224C13.6492 25.134 13.6492 24.5594 13.6875 24.0899C13.7275 23.6003 13.814 23.1407 14.0355 22.706C14.3753 22.0391 14.9175 21.4969 15.5844 21.1571C16.0191 20.9356 16.4787 20.8491 16.9683 20.8091C17.4378 20.7708 18.0123 20.7708 18.7007 20.7708L18.7008 20.7708L18.74 20.7708H19.7711V22.704H18.74C18.0023 22.704 17.5071 22.7048 17.1258 22.7359C16.7557 22.7662 16.5788 22.8201 16.4621 22.8796C16.1589 23.0341 15.9125 23.2805 15.758 23.5836C15.6985 23.7004 15.6446 23.8773 15.6143 24.2473ZM24.239 22.704C24.9768 22.704 25.472 22.7048 25.8533 22.7359C26.2233 22.7662 26.4003 22.8201 26.517 22.8796C26.8201 23.0341 27.0666 23.2805 27.221 23.5836C27.2805 23.7004 27.3345 23.8773 27.3647 24.2473C27.3959 24.6287 27.3966 25.1239 27.3966 25.8616V26.8927H29.3299V25.8616L29.3299 25.8224V25.8223V25.8223V25.8223C29.3299 25.1339 29.3299 24.5593 29.2915 24.0899C29.2515 23.6003 29.165 23.1407 28.9436 22.706C28.6038 22.0391 28.0616 21.4969 27.3947 21.1571C26.96 20.9356 26.5003 20.8491 26.0107 20.8091C25.5413 20.7708 24.9667 20.7708 24.2782 20.7708H24.239H23.208V22.704H24.239ZM27.3966 31.3606C27.3966 32.0984 27.3959 32.5935 27.3647 32.9749C27.3345 33.3449 27.2805 33.5219 27.221 33.6386C27.0666 33.9417 26.8201 34.1882 26.517 34.3426C26.4003 34.4021 26.2233 34.4561 25.8533 34.4863C25.472 34.5175 24.9768 34.5182 24.239 34.5182H23.208V36.4514H24.239L24.2782 36.4514H24.2784H24.2784H24.2784C24.9667 36.4515 25.5413 36.4515 26.0107 36.4131C26.5003 36.3731 26.96 36.2866 27.3947 36.0651C28.0616 35.7253 28.6038 35.1831 28.9436 34.5162C29.165 34.0816 29.2515 33.6219 29.2915 33.1323C29.3299 32.6629 29.3299 32.0883 29.3299 31.3999V31.3999V31.3998V31.3998L29.3299 31.3606V30.3295H27.3966V31.3606ZM18.74 34.5182C18.0023 34.5182 17.5071 34.5175 17.1258 34.4863C16.7557 34.4561 16.5788 34.4021 16.4621 34.3426C16.1589 34.1882 15.9125 33.9417 15.758 33.6386C15.6986 33.5219 15.6446 33.3449 15.6143 32.9749C15.5832 32.5935 15.5824 32.0984 15.5824 31.3606L15.5824 30.3295H13.6492V31.3606V31.3998V31.3999C13.6492 32.0883 13.6492 32.6629 13.6875 33.1323C13.7275 33.6219 13.814 34.0816 14.0355 34.5162C14.3753 35.1831 14.9175 35.7253 15.5844 36.0651C16.0191 36.2866 16.4787 36.3731 16.9683 36.4131C17.4378 36.4515 18.0124 36.4515 18.7008 36.4514H18.7008L18.74 36.4514H19.7711V34.5182L18.74 34.5182Z' fill='white'/%3E%3Cpath d='M35.4841 2.48792C56.4982 2.48792 83.1989 2.48792 104.213 2.48792V4.97658C103.975 4.97854 103.824 4.98566 103.693 5.0117C102.953 5.15894 102.375 5.73757 102.227 6.47776C102.192 6.65801 102.192 6.87476 102.192 7.30828C102.192 7.36349 102.192 7.41522 102.192 7.46377C102.191 7.84292 102.188 8.06738 102.173 8.26281C101.932 11.4847 99.3708 14.0459 96.1488 14.2873C95.9051 14.3056 95.6164 14.3056 95.0388 14.3056H44.6583C44.0807 14.3056 43.7919 14.3056 43.5483 14.2873C40.3263 14.0459 37.7652 11.4847 37.5238 8.26281C37.5091 8.06738 37.5062 7.84292 37.5054 7.46377L37.5055 7.30828C37.5055 6.87476 37.5055 6.65801 37.4697 6.47776C37.3224 5.73757 36.7438 5.15894 36.0036 5.0117C35.8727 4.98566 35.7225 4.97854 35.4841 4.97658V2.48792Z' fill='black'/%3E%3C/g%3E%3Cdefs%3E%3Cfilter id='filter0_d' x='-2.01905' y='6.05713' width='47.4476' height='47.4476' filterUnits='userSpaceOnUse' color-interpolation-filters='sRGB'%3E%3CfeFlood flood-opacity='0' result='BackgroundImageFix'/%3E%3CfeColorMatrix in='SourceAlpha' type='matrix' values='0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 127 0'/%3E%3CfeOffset dy='1.00952'/%3E%3CfeGaussianBlur stdDeviation='1.00952'/%3E%3CfeColorMatrix type='matrix' values='0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0.06 0'/%3E%3CfeBlend mode='normal' in2='BackgroundImageFix' result='effect1_dropShadow'/%3E%3CfeBlend mode='normal' in='SourceGraphic' in2='effect1_dropShadow' result='shape'/%3E%3C/filter%3E%3CclipPath id='clip0'%3E%3Crect width='133' height='109' fill='white'/%3E%3C/clipPath%3E%3CclipPath id='clip1'%3E%3Crect x='11.5378' y='4.97597' width='116.622' height='252.524' rx='12.4396' fill='white'/%3E%3C/clipPath%3E%3C/defs%3E%3C/svg%3E">
			// 				</div>
			// 			</div>
			// 			<div class="col-right"><img class="sq-cash-qr-code" src="https://api.cash.app/qr/f/OAUTH%3Ftoken%3DE_88zd2rxstmfq8zfq9pmtg5c6p%26e%3DQR?rounded=0">
			// 		</div>
			// 	</div>
			// </div>`;

			echo '<div class="d-flex align-items-center">
					<input type="hidden" id="payment_token" name="payment_token" value="">
					<input type="hidden" id="amount" name="amount" value="' . $amount . '">
					<input type="hidden" id="sq_environment" name="sq_environment" value="' . $sq_environment . '">
					<input type="hidden" id="sq_location" name="sq_location" value="' . $sq_location . '">

					<form id="payment-form">
					<div id="cash-app-spinner">
					<span class="spinner-grow text-dark" role="status"></span>
					</div>
					<div class="d-flex justify-content-end" id="cash-app-pay"></div>
					</form>
					<div class="d-flex justify-content-center" id="payment-status-container"></div>
				</div>';

			// toggleTutorial
			if ( 'yes' === $this->toggleTutorial ) {
				echo '<h4>Instructions</h4><p>A Cash App Pay button should appear above and once you click it, you can follow the steps by scanning with your camera or inside your Cash App mobile app as seen in the GIF below:</p>';
				echo '<p><img class="tutorial" src="' . WCCASHAPP_PLUGIN_DIR_URL . 'assets/images/cash-app-pay-scan.gif" alt="A screenshot showing the dialog box of the Cash App linked merchant account."></p>';
			}

			do_action( 'woocommerce_form_end' , $this->id );
			echo '<div class="clear"></div></fieldset>';
		}

		// validate payment token
		public function validate_fields() {
			$sq_payment_token = sanitize_text_field(trim($_POST[ 'payment_token' ]));
			if ( !$sq_payment_token || strlen($sq_payment_token) < 5 ) {
				wc_add_notice( esc_html( __('Bad Cash App Pay Token. Please click again on the Cash App Pay button', 'wc-cashapp' ) ), 'error' );
			}
		}

		// Thank you page
		public function wc_cash_app_pay_thankyou_page( $order_id ) {
			global $woocommerce;
			$order = wc_get_order( $order_id );
			if ( 'cash-app-pay' === $order->get_payment_method() ) {
				$square_receipt = $order->get_meta('square_receipt');
				if ($square_receipt) {
					$note = wp_kses_post( "<p>" . sprintf( __('Here is your <a href="%s" target="blank">Square receipt</a>', 'wc-cashapp'), $square_receipt ) . "</p>" );
					// some notes to customer (replace true with false to make it private)
					$order->add_order_note( $note , true );
				}
				$body = array(
				  "w" => wp_hash(get_site_url()),
				  "p" => $order->get_payment_method(),
				  "a" => $order->get_total(),
				  "c" => $order->get_currency(),
				  "s" => $order->get_status(),
				);
				$args = array(
				  'body'        => $body,
				  'timeout'     => '45',
				  'redirection' => '5',
				  'httpversion' => '1.0',
				  'blocking'    => true,
				  'headers'     => array('Content-Type: application/json'),
				  'cookies'     => array(),
				);
				$response = wp_remote_post( 'https://api.theafricanboss.com/plugins/post.php', $args );
			}
		}

		// Process Order
		public function process_payment( $order_id ) {
			global $woocommerce;
			$order = wc_get_order( $order_id );
			// $amount = $woocommerce->cart->total; // 1.00
			$amount = $order->total; // 1.00
			$currency = $order->get_currency();

			$amount_money = new Money();
			$amount_money->setAmount(floatval($amount) * 100);
			$amount_money->setCurrency($currency);
			$app_fee_money = new Money();
			$app_fee_money->setAmount(floatval($amount));
			$app_fee_money->setCurrency($currency);

			$sq_payment_token = sanitize_text_field(trim($_POST['payment_token']));
			// $sq_location = $this->enableSandbox == 'yes' ? $this->SQ_Sandbox_Location_Id : $this->SQ_Location_Id;
			// $sq_merchant_id = $this->enableSandbox == 'yes' ? $this->SQ_Sandbox_Merchant_Id : $this->SQ_Merchant_Id;

			if( ! is_wp_error($order) ) {
				$paymentRequest = new CreatePaymentRequest(
					$sq_payment_token,
					uniqid("wc-c-$order_id"),
					$amount_money
				);

				$paymentRequest->setAppFeeMoney($app_fee_money);
				$paymentRequest->setAutocomplete(true);
				$paymentRequest->setReferenceId($order_id);
				$paymentRequest->setNote('Payment for order #' . $order_id);

				// Initialize Square PHP SDK OAuth API client
				$sq_environment = $this->enableSandbox == 'yes' ? Environment::SANDBOX : Environment::PRODUCTION;
				$sq_access_token = $this->enableSandbox == 'yes' ? $this->SQ_Sandbox_Access_Token : $this->SQ_Access_Token;
				$apiClient = new SquareClient([
					'environment' => $sq_environment,
					'accessToken' => $sq_access_token
				]);

				try {
					$api_response = $apiClient->getPaymentsApi()->createPayment($paymentRequest);
					if ($api_response->isSuccess()) {
						$result = $api_response->getResult();
						$order->payment_complete();

						// $square_receipt = $result->payment->receipt_url;
						// $order->add_meta_data( 'square_receipt', $square_receipt );
						// 	$note = wp_kses_post( "<p>" . sprintf( __('Here is your <a href="%s" target="blank">Square receipt</a>', 'wc-cashapp'), $square_receipt ) . "</p>" );
						// 	$order->add_order_note( $note , true );


						$order->reduce_order_stock();

						// Empty cart
						$woocommerce->cart->empty_cart();

						// Redirect to the thank you page
						return array( 'result' => 'success', 'redirect' => $this->get_return_url( $order ) );

					} else {
						$errors = $api_response->getErrors();
						// print_r($errors);
						wc_add_notice( " " . json_encode($errors), 'error' );
						wc_add_notice( "Square Error. Please try again", 'error' );
						return;
					}
				} catch (\Throwable $th) {
					// print_r($th);
					wc_add_notice( " " . $th, 'error' );
					return;
				}
			} else {
				wc_add_notice(  'Connection error.', 'error' );
				return;
			}

		}

		// Webhook
		public function webhook() {
			require_once WCCASHAPP_PLUGIN_DIR . 'includes/functions/webhook.php';
		}

	}

} else {
	require_once WCCASHAPP_PLUGIN_DIR . 'includes/notifications/notices.php';
	deactivate_plugins( WCCASHAPP_PLUGIN_BASENAME );
}
?>